<?php
namespace App;

use DI\ContainerBuilder;
use Interop\Container\ContainerInterface;

class App extends \DI\Bridge\Slim\App {

    protected function configureContainer(ContainerBuilder $builder)
    {
        $definitions = [
            'settings.displayErrorDetails' => true,

            \Doctrine\DBAL\Connection::class => function () {
                
                if(!file_exists(__DIR__ . '/../environment/environment.json')) {
                    throw new \Exception("environment not found!");
                }

                $environmentFile = file_get_contents(__DIR__ . '/../environment/environment.json');
                $environment     = json_decode($environmentFile, true);

                $connection = null;

                if($environment['driver'] == 'oci8') {
                    
                    $dbPassword = $environment['password'];
                    if(isset($environment['encryptedPassword']) && $environment['encryptedPassword']) {
                        $dbPassword = $this->encrypt_decrypt('decrypt', $environment['password']);
                    }
                    
                    $connection = \Doctrine\DBAL\DriverManager::getConnection(array(
                        'user'        => $environment['user'],
                        'password'    => $dbPassword,
                        'host'        => $environment['host'],
                        'port'        => $environment['port'],
                        'dbname'      => $environment['dbname'],
                        'service'     => $environment['service'],
                        'driver'      => $environment['driver'],
                        'charset'     => $environment['charset'],
                        'sessionMode' => $environment["sessionMode"] === "OCI_CRED_EXT" ? OCI_CRED_EXT : OCI_DEFAULT
                    ), $config = new \Doctrine\DBAL\Configuration);
                } else if ($environment['driver'] == 'sqlsrv') {
                    $connection = \Doctrine\DBAL\DriverManager::getConnection(array(
                        'user'     => $environment['user'],
                        'password' => $environment['password'],
                        'host'     => $environment['host'],
                        'port'     => $environment['port'],
                        'dbname'   => $environment['dbname'],
                        'driver'   => $environment['driver']
                    ), $config = new \Doctrine\DBAL\Configuration);
                }
                return $connection;
            }
        ];

        $builder->addDefinitions($definitions);
    }
    
    private function encrypt_decrypt($action, $string) {
        $output = false;
        $encrypt_method = "AES-256-CBC";
    
        $secret_key = 'ZEEDHI';
        $secret_iv  = 'ZEEDHI IV';
    
        // hash
        $key = hash('sha256', $secret_key);
        
        // iv - encrypt method AES-256-CBC expects 16 bytes - else you will get a warning
        $iv = substr(hash('sha256', $secret_iv), 0, 16);
        if ( $action == 'encrypt' ) {
            $output = openssl_encrypt($string, $encrypt_method, $key, 0, $iv);
            $output = base64_encode($output);
        } else if( $action == 'decrypt' ) {
            $output = openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
        }
        return $output;
    }
}
