<?php

namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class DadosVenda {

    protected $dadosVendaService;

    /**
     * Construtor
     *
     * @param \App\Service\DadosVenda $dadosVendaService
     */
    public function __construct(\App\Service\DadosVenda $dadosVendaService){
        $this->dadosVendaService = $dadosVendaService;
    }
    
    public function welcome(Request $request, Response $response) {
        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date']
        ]);
    }

    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    public function recuperaDadosVenda(Request $request, Response $response) {
   	    try{
   	        $input   = $request->getParsedBody();
            $periodo = $input['periodo'];
            $inicio  = $periodo['inicio'];
            $fim     = $periodo['final'];
            $dadosVenda = $this->dadosVendaService->recuperaDadosVenda($inicio, $fim);
            $arrayRetorno = $this->dadosVendaService->formataDadosVenda($dadosVenda);
            return $response->withJson(
                array(
                    'error' => false,
                    'message'=> "",
                    'data' => $arrayRetorno
                )
            );
        }catch(\Exception $e){
            return $response->withJson(
                array(
                    'error' => true,
                    'message' => $e->getMessage(),
                    'data' => array()
                )
            );
        }
   	}
  
    /**
     * Recupera todos os dados das vendas no período informado e retorna um Array de objetos JSON mostrando os resultados.
     *
     * @param Request $request
     * @param Response $response
     * @return void
     */
   	public function recuperaDadosVendaGrupo(Request $request, Response $response) {
   	    try{
            $input   = $request->getParsedBody();
            $periodo = $input['periodo'];
            $inicio  = $periodo['inicio'];
            $fim     = $periodo['final'];
            $dadosVenda = $this->dadosVendaService->recuperaDadosVendaGrupo($inicio, $fim);
            foreach($dadosVenda as &$venda){
                $movimentacoes = $this->dadosVendaService->recuperaMovimentacoesVenda($venda['CDFILIAL'], $venda['CDCAIXA'], $venda['NRSEQVENDA']);
                $venda['MOVIMENTACOES'] = $movimentacoes;
                $gruposItens = $this->dadosVendaService->recuperaGruposItensVenda($venda['CDFILIAL'], $venda['CDCAIXA'], $venda['NRSEQVENDA']);
                $venda['GRUPOS'] = $gruposItens;
                foreach($venda['GRUPOS'] as &$grupo){
                    $itens = $this->dadosVendaService->recuperaItensGrupoVenda($venda['CDFILIAL'], $venda['CDCAIXA'], $venda['NRSEQVENDA'], $grupo['NMSUBGRPROD']);
                    $grupo['ITENS'] = $itens;
                }
                unset($grupo);
            }
            unset($venda);
            $arrayRetorno = $this->dadosVendaService->formataVendasGrupo($dadosVenda);
            return $response->withJson(
                array(
                    'error' => false,
                    'message'=> "",
                    'data' => $arrayRetorno
                )
            );
        }catch(\Exception $e){
            return $response->withJson(
                array(
                    'error' => true,
                    'message' => $e->getMessage(),
                    'data' => array()
                )
            );
        }
   	}
   	
   	public function recuperaDadosRelatorio(Request $request, Response $response){
        try{
            $input   = $request->getParsedBody();
            $periodo = $input['periodo'];
            $filiais = !empty($input['filiais']) ? $input['filiais'] : array('T');
            $inicio  = $periodo['inicio'];
            $fim     = $periodo['final'];
            
            $relatorio = $this->dadosVendaService->recuperaRelatorio($inicio, $fim, $filiais);
            $relatorioFormatado = $this->dadosVendaService->formataRelatorio($relatorio);
            
            return $response->withJson(
                array(
                    'error' => false,
                    'message'=> "",
                    'data' => $relatorioFormatado
                )
            );
        }catch(\Exception $e){
            return $response->withJson(
                array(
                    'error' => true,
                    'message' => $e->getMessage(),
                    'data' => array()
                )
            );
        }
   	}
   
}