<?php

namespace App\Factory;

class DadosVenda{
    
    private $connection;
    
    public function __construct(\Doctrine\DBAL\Connection $connection){
        $this->connection = $connection;
    }
    
    public function recuperaDadosVenda($inicio, $fim){
        $sql = "
        SELECT DTENTRVENDA, GRUPO, FILIAL, TIPOVENDA,
               SUM(PESSOAS) AS PESSOAS, SUM(TOTAL) AS TOTAL
          FROM (SELECT TO_CHAR(V.DTENTRVENDA, 'DD/MM/YYYY') DTENTRVENDA,
                       FI.NMFILIAL as GRUPO,
                       FI.NMFILIAL AS FILIAL,
                       V.CDCAIXA,
                       V.NRSEQVENDA,
                       CASE SUBSTR(V.IDORIGEMVENDA, 1, 3)
                            WHEN 'MES' THEN 'Mesa'
                            ELSE 'Ticket' END AS TIPOVENDA,
                       NVL(V.NRPESMESAVENDA,1) AS PESSOAS,
                       SUM((IV.QTPRODVEND*(IV.VRUNITVEND+NVL(IV.VRUNITVENDCL,0))) + NVL(IV.VRACRITVEND,0) - NVL(IV.VRDESITVEND,0)) TOTAL
                  FROM VENDA V, ITEMVENDA IV, FILIAL FI
                 WHERE V.DTENTRVENDA BETWEEN TO_DATE(:P_DTINI, 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:P_DTFIN , 'DD/MM/YYYY HH24:MI:SS')
                   AND V.CDFILIAL    = IV.CDFILIAL
                   AND V.CDCAIXA     = IV.CDCAIXA
                   AND V.NRSEQVENDA  = IV.NRSEQVENDA
                   AND V.CDFILIAL    = FI.CDFILIAL
                   AND V.IDSITUVENDA = 'O'   
                 GROUP BY TO_CHAR(V.DTENTRVENDA, 'DD/MM/YYYY') , FI.NMFILIAL, V.NRSEQVENDA,SUBSTR(V.IDORIGEMVENDA, 1, 3), V.CDCAIXA, NRPESMESAVENDA)
           GROUP BY DTENTRVENDA, GRUPO, FILIAL,  TIPOVENDA
           ORDER BY DTENTRVENDA, FILIAL, TIPOVENDA
        ";
        
        $parametros = array(
            "P_DTINI" => $inicio,
            "P_DTFIN" => $fim
        );
        
        $dadosVendas = $this->connection->fetchAll($sql, $parametros);
        
        return !empty($dadosVendas) ? $dadosVendas : array();
    }
    
    public function recuperaVendasPeriodo($inicio, $fim){
        $sql = "
            SELECT DTENTRVENDA, GRUPO, FILIAL, CDFILIAL, NMCAIXA, CDCAIXA, TIPOVENDA, NMOPERADOR,
                   CDOPERADOR, IDORIGEMVENDA, NRSEQVENDA,
                   SUM(PESSOAS) AS PESSOAS, SUM(TOTAL) AS TOTAL
              FROM (SELECT TO_CHAR(V.DTENTRVENDA, 'DD/MM/YYYY') DTENTRVENDA,
                           FI.NMFILIAL as GRUPO,
                           FI.NMFILIAL AS FILIAL,
                           FI.CDFILIAL,
                           CX.NMCAIXA,
                           V.CDCAIXA,
                           V.NRSEQVENDA,
                           OP.NMOPERADOR,
                           OP.CDOPERADOR,
                           SUBSTR(V.IDORIGEMVENDA, 1, 3) IDORIGEMVENDA,
                           CASE SUBSTR(V.IDORIGEMVENDA, 1, 3)
                              WHEN 'MES' THEN 'Mesa'
                              ELSE 'Ticket' END AS TIPOVENDA,
                           NVL(V.NRPESMESAVENDA,1) AS PESSOAS,
                           SUM((IV.QTPRODVEND*(IV.VRUNITVEND+NVL(IV.VRUNITVENDCL,0))) + NVL(IV.VRACRITVEND,0) - NVL(IV.VRDESITVEND,0)) TOTAL
                      FROM VENDA V, ITEMVENDA IV, FILIAL FI, CAIXA CX, OPERADOR OP
                     WHERE V.DTENTRVENDA BETWEEN TO_DATE(:P_DTINI, 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:P_DTFIN , 'DD/MM/YYYY HH24:MI:SS')
                       AND V.CDFILIAL = IV.CDFILIAL
                       AND V.CDCAIXA = IV.CDCAIXA
                       AND V.NRSEQVENDA = IV.NRSEQVENDA
                       AND V.CDFILIAL = FI.CDFILIAL
                       AND V.IDSITUVENDA = 'O'   
                       AND V.CDCAIXA = CX.CDCAIXA
                       AND V.CDFILIAL = CX.CDFILIAL
                       AND OP.CDOPERADOR = V.CDOPERADOR
                  GROUP BY TO_CHAR(V.DTENTRVENDA, 'DD/MM/YYYY'), FI.NMFILIAL, FI.CDFILIAL, CX.NMCAIXA, V.CDCAIXA, V.NRSEQVENDA, SUBSTR(V.IDORIGEMVENDA, 1, 3), NMOPERADOR, OP.CDOPERADOR, NRPESMESAVENDA
                  ORDER BY DTENTRVENDA, FI.NMFILIAL, FI.CDFILIAL, CX.NMCAIXA, V.CDCAIXA, SUBSTR(V.IDORIGEMVENDA, 1, 3), NMOPERADOR, OP.CDOPERADOR, NRPESMESAVENDA)
            GROUP BY DTENTRVENDA, GRUPO, FILIAL, CDFILIAL, NMCAIXA, CDCAIXA, NRSEQVENDA, TIPOVENDA, NMOPERADOR, CDOPERADOR, IDORIGEMVENDA
            ORDER BY DTENTRVENDA, GRUPO, FILIAL, NMCAIXA, NMOPERADOR
        ";
        
        $parametros = array(
            "P_DTINI" => $inicio,
            "P_DTFIN" => $fim
        );
        
        $dadosVendas = $this->connection->fetchAll($sql, $parametros);
        
        return !empty($dadosVendas) ? $dadosVendas : array();
    }
    
    public function recuperaMovimentacoesVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA){
        $sql = "
            SELECT IDTIPOMOVIVE, VRMOVIVEND, MV.CDTIPORECE, TP.NMTIPORECE
              FROM MOVCAIXA MV
              JOIN TIPORECE TP
                ON TP.CDTIPORECE = MV.CDTIPORECE
             WHERE CDFILIAL = :CDFILIAL
               AND CDCAIXA = :CDCAIXA
               AND NRSEQVENDA = :NRSEQVENDA
        ";
        $parametros = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA
        );
        $movimentacoes = $this->connection->fetchAll($sql, $parametros);
        return !empty($movimentacoes) ? $movimentacoes : array();
    }
    
    public function recuperaGruposItensVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA){
        $sql = "
            SELECT NMSUBGRPROD 
            FROM ITEMVENDA IV, PRODUTO P, PRSUBGRPROD PR, SUBGRUPPROD SG 
           WHERE IV.CDFILIAL = :CDFILIAL 
             AND IV.CDCAIXA = :CDCAIXA 
             AND IV.NRSEQVENDA = :NRSEQVENDA
             AND P.CDPRODUTO = IV.CDPRODUTO
             AND P.CDPRODUTO = PR.CDPRODUTO
             AND PR.CDGRUPPROD = SG.CDGRUPPROD 
             AND PR.CDSUBGRPROD = SG.CDSUBGRPROD 
           GROUP BY NMSUBGRPROD
        ";
        $parametros = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA
        );
        $itens = $this->connection->fetchAll($sql, $parametros);
        return !empty($itens) ? $itens : array();
    }
    
    public function recuperaItensGrupoVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NMSUBGRPROD){
        $sql = "
            SELECT P.CDPRODUTO, NMPRODUTO 
              FROM ITEMVENDA IV, PRODUTO P, PRSUBGRPROD PR, SUBGRUPPROD SG 
             WHERE IV.CDFILIAL = :CDFILIAL 
               AND IV.CDCAIXA = :CDCAIXA
               AND IV.NRSEQVENDA = :NRSEQVENDA
               AND P.CDPRODUTO = IV.CDPRODUTO
               AND P.CDPRODUTO = PR.CDPRODUTO
               AND PR.CDGRUPPROD = SG.CDGRUPPROD
               AND PR.CDSUBGRPROD = SG.CDSUBGRPROD
               AND NMSUBGRPROD = :NMSUBGRPROD 
             ORDER BY NRSEQUITVEND
        ";
        $parametros = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA,
            'NMSUBGRPROD' => $NMSUBGRPROD
        );
        $itens = $this->connection->fetchAll($sql, $parametros);
        return !empty($itens) ? $itens : array();
    }
    
    public function recuperarTurnos($inicio, $fim, $filiais){
        $sql = "
            SELECT TC.CDFILIAL, (TC.CDFILIAL ||' - '|| FI.NMFILIAL) FILIAL,
                   (OP.CDOPERADOR ||' - '||OP.NMOPERADOR) OPERADOR,
                   TO_CHAR(TC.DTABERCAIX, 'DD/MM/YYYY HH24:MI:SS') DTABERCAIX,
                   TO_CHAR(TC.DTFECHCAIX, 'DD/MM/YYYY HH24:MI:SS') DTFECHCAIX, TC.CDCAIXA
              FROM TURCAIXA TC
              JOIN OPERADOR OP
                ON OP.CDOPERADOR = TC.CDOPERADOR
              JOIN FILIAL FI
                ON FI.CDFILIAL = TC.CDFILIAL
             WHERE ((TC.CDFILIAL IN (:CDFILIAL)) OR 'T' IN (:CDFILIAL))
               AND DTABERCAIX BETWEEN TO_DATE(:DTINICIO, 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:DTFIM, 'DD/MM/YYYY HH24:MI:SS')
               AND DTFECHCAIX IS NOT NULL
          ORDER BY TC.CDFILIAL, DTABERCAIX
        ";
         $parametros = array(
            "DTINICIO" => $inicio,
            "DTFIM" => $fim,
            "CDFILIAL" => $filiais
        );
        
        $tipos = array(
            "CDFILIAL" => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY
        );
        
        $turnos = $this->connection->fetchAll($sql, $parametros, $tipos);
        return !empty($turnos) ? $turnos : array();
    }
    
    public function recuperaPagamentos($cdfilial, $cdcaixa, $dtinicio, $dtfim){
        $sql = "
            SELECT MV.CDFILIAL, MV.CDCAIXA,
                   SUM(VRMOVIVEND) SOMA, IDTIPOMOVIVE, MV.CDTIPORECE, NMTIPORECE 
              FROM MOVCAIXA MV
              JOIN TIPORECE TP
                ON TP.CDTIPORECE = MV.CDTIPORECE
             WHERE CDFILIAL = :CDFILIAL
               AND CDCAIXA = :CDCAIXA
               AND DTHRINCMOV BETWEEN TO_DATE(:DTINICIO, 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:DTFIM, 'DD/MM/YYYY HH24:MI:SS')
               AND IDTIPOMOVIVE <> 'A'
             GROUP BY MV.CDFILIAL, MV.CDCAIXA, IDTIPOMOVIVE, MV.CDTIPORECE, NMTIPORECE
             ORDER BY MV.CDTIPORECE, IDTIPOMOVIVE
        ";
        $parametros = array(
            "DTINICIO" => $dtinicio,
            "DTFIM" => $dtfim,
            "CDFILIAL" => $cdfilial,
            "CDCAIXA" => $cdcaixa
        );
        $pagamentos = $this->connection->fetchAll($sql, $parametros);
        return !empty($pagamentos) ? $pagamentos : array();
    }
    
    public function recuperaVendasTurno($cdfilial, $cdcaixa, $dtinicio, $dtfim){
        $sql = "
            SELECT VE.CDFILIAL, VE.CDCAIXA, CASE SUBSTR(VE.IDORIGEMVENDA, 1, 3)
                        WHEN 'MES' THEN 'Mesa'
                        ELSE 'Ticket' END AS TIPOVENDA,
                   NVL(VE.NRPESMESAVENDA,1) AS PESSOAS,
                   SUM(VRTOTVENDA) VRTOTVENDA,
                   COUNT(*) QUANTVENDAS
              FROM VENDA VE
             WHERE VE.CDFILIAL = :CDFILIAL
               AND VE.CDCAIXA = :CDCAIXA
               AND DTVENDA BETWEEN TO_DATE(:DTINICIO, 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:DTFIM, 'DD/MM/YYYY HH24:MI:SS')
               GROUP BY VE.CDFILIAL, VE.CDCAIXA, SUBSTR(VE.IDORIGEMVENDA, 1, 3), NVL(VE.NRPESMESAVENDA,1)
        ";
        $parametros = array(
            "CDFILIAL" => $cdfilial,
            "CDCAIXA" => $cdcaixa,
            "DTINICIO" => $dtinicio,
            "DTFIM" => $dtfim
        );
        $pagamentos = $this->connection->fetchAll($sql, $parametros);
        return !empty($pagamentos) ? $pagamentos : array();
    }
    
}